#include "stdafx.h"
#include "s_comm.h"

unsigned char HammingCode[16] = 
	{
		0x00 , 0x0F , 0x13 , 0x1C ,
		0x25 , 0x2A , 0x36 , 0x39 ,
		0x46 , 0x49 , 0x55 , 0x5A ,
		0x63 , 0x6C , 0x70 , 0x7F };


void Convert2Code(unsigned char input, unsigned char & p_low, unsigned char & p_high)
{
	p_low = HammingCode[input & 0x0F];
	p_high = HammingCode[(input>>4) & 0x0F];
	return;
}

char Convert2Byte(unsigned char p_low, unsigned char p_high)
{
	p_high&= 0x0F;
	p_low&= 0x0F;
	p_high <<= 4;
	return (p_high + p_low);
}

//prototype
char A_CodeToData (unsigned char code);
char CodeToData (unsigned char code);

char CorrectAndDecode(unsigned char code)
{
	unsigned char mask = 1;
	unsigned char n;
	char i;
	for (i = 0; i < 8; i++) 
	{
		n = A_CodeToData (code ^ mask);
		if (n != -1) 
			// Corrected it!
			return n;
		mask <<= 1;
	}
	// did not work
	return -1;
}

char A_CodeToData (unsigned char code)
{
	char i;
	for (i = 0; i < 16; i++)
	{
		if (code == HammingCode[i])
		{
			return i;
		}
	}
	// Not a code!
	return -1;
}

char CodeToData(unsigned char code)
{
	char n;
	n = A_CodeToData(code);
	if(n != -1)
		return n;
	n = CorrectAndDecode(code);
	return n;
}

void UnHamming(unsigned char * szBuff, unsigned char chars_read)
{
	unsigned char first_half;
	unsigned char i;
	for(i=0;i<chars_read;i++)
	{
		if(i%2)
			szBuff[i/2]=Convert2Byte(first_half,CodeToData(szBuff[i]));
		else
			first_half = CodeToData(szBuff[i]);
	}
	szBuff[i/2]=0;
}


CString ConvertError(DWORD p_msgid)
{
	LPVOID lpMsgBuf;
	CString retval;
	FormatMessage(
		FORMAT_MESSAGE_ALLOCATE_BUFFER | 
		FORMAT_MESSAGE_FROM_SYSTEM | 
		FORMAT_MESSAGE_IGNORE_INSERTS,
		NULL,
		p_msgid,
		0, // Default language
		(LPTSTR) &lpMsgBuf,
		0,
		NULL 
		);
	retval=(LPCTSTR)lpMsgBuf;
	LocalFree( lpMsgBuf );
	return retval;
}

S_comm::S_comm(void)
{
	hSerial=NULL;
}

S_comm::~S_comm(void)
{
	if(hSerial)
		CloseHandle(hSerial);
	hSerial=NULL;
}

int S_comm::Open(CString & error, short port_number)
{
	CString port_name;
	port_name.Format("COM%d:",port_number);
	//port_name="COM11:";
	hSerial = CreateFile(port_name.GetBuffer(),
		GENERIC_READ | GENERIC_WRITE,
		0,
		0,
		OPEN_EXISTING,
		FILE_ATTRIBUTE_NORMAL,
		0);
	if(hSerial==INVALID_HANDLE_VALUE)
	{
		error+="Error creating file for port.\r\n";
		error+=ConvertError(GetLastError());
		hSerial=NULL;
		return -1;
	}
	DCB dcbSerialParams = {0};
	dcbSerialParams.DCBlength=sizeof(dcbSerialParams);
	if (!GetCommState(hSerial, &dcbSerialParams)) 
	{
		error+="Error getting state\r\n";
		error+=ConvertError(GetLastError())+"\r\n";
		return -1;
	}
	dcbSerialParams.BaudRate=CBR_57600;
	dcbSerialParams.ByteSize=8;
	dcbSerialParams.StopBits=ONESTOPBIT;
	dcbSerialParams.Parity=NOPARITY;
	if(!SetCommState(hSerial, &dcbSerialParams))
	{
		error+="Error setting state.\r\n";
		error+=ConvertError(GetLastError())+"\r\n";
		return -1;
	}

	COMMTIMEOUTS timeouts={0};
	timeouts.ReadIntervalTimeout=80;
	timeouts.ReadTotalTimeoutConstant=50;
	timeouts.ReadTotalTimeoutMultiplier=10;
	timeouts.WriteTotalTimeoutConstant=50;
	timeouts.WriteTotalTimeoutMultiplier=10;
	if(!SetCommTimeouts(hSerial, &timeouts))
	{
		error+="Error setting timeouts.\r\n";
		error+=ConvertError(GetLastError())+"\r\n";
		return -1;
	}
	error+="Com port opened\r\n";
	return 0;
}

int S_comm::Close(CString & error)
{
	if(hSerial && (hSerial!=INVALID_HANDLE_VALUE))
	{
		CloseHandle(hSerial);
		error+="Port closed\r\n";
	}
	else
		error+="Port was not open\r\n";
	hSerial=NULL;
	return 0;
}

int S_comm::Read(unsigned char * szBuff, CString & error)
{
	if(!hSerial || hSerial==INVALID_HANDLE_VALUE)
	{
		error+="Com port not open at read\r\n";
		return -1;
	}
	//r_string = "";
	DWORD dwBytesRead = 0;
	if(!ReadFile(hSerial, szBuff, 8, &dwBytesRead, NULL))
	{
		error+="Error when reading from serial port.\r\n";
		return -1;
	}
	return dwBytesRead;
}

int S_comm::Write(unsigned char * buffer, unsigned char length, CString & error)
{
	CString s_temp="",s2temp;
	if(!hSerial || hSerial==INVALID_HANDLE_VALUE)
	{
		error+="Com port not open at write\r\n";
		return -1;
	}
	DWORD dwBytesWritten=0;
	if(!WriteFile(hSerial, buffer, length, &dwBytesWritten, NULL))
	{
		error+=ConvertError(GetLastError());
		error+="Error when writing to serial port.\r\n";
		return -1;
	}
	for(int i=0;i<length;i++)
	{
		s2temp.Format("%d",buffer[i]);
		s_temp += s2temp+" ";
	}
	error+=s_temp+" has been written (";
	s_temp.Format("%d bytes)\r\n",dwBytesWritten);
	error+=s_temp;
	return dwBytesWritten;
}

int S_comm::Status()
{
	if(!hSerial || hSerial==INVALID_HANDLE_VALUE)
	{
		return FALSE;
	}
	return TRUE;
}